#pragma semicolon 1

#include <sourcemod>
#include <sdkhooks>
#include <vip_old>

public Plugin:myinfo = 
{
	name	= "VIP [Damage]",
	author	= "wS",
	version = "1.2.2"
};

#define VipModule_COUNT 6

enum VipModule
{
	dmg_fall = 0,	//   
	dmg_world,		// 
	dmg_team,		//   
	dmg_self,		//  
	dmg_attack,		//   
	dmg_protect		//    
};

new const String:g_sItemName[][] = 
{
	"dmg_fall",
	"dmg_world",
	"dmg_team",
	"dmg_self",
	"dmg_attack",
	"dmg_protect"
};

new g_bAccess[MAXPLAYERS + 1][VipModule_COUNT];
new g_ItemID[VipModule_COUNT];

public OnPluginStart()
{
	//
	new bool:ok = false;
	new Handle:kv = CreateKeyValues("damage");
	if (FileToKeyValues(kv, "cfg/vip/modules/damage/damage.txt"))
	{
		for (new module = -1; ++module < VipModule_COUNT;)
		{
			if (KvGetNum(kv, g_sItemName[module]) == 1)
			{
				if (!(g_ItemID[module] = VIP_RegisterItem(g_sItemName[module], VIP_ITEM_TOGGLE))) {
					SetFailState("VIP_RegisterItem error (%s)", g_sItemName[module]);
				}
				
				ok = true;
			}
		}
	}
	CloseHandle(kv);
	if (!ok) SetFailState("All dmg_ 0 (check cfg/vip/modules/damage/damage.txt)");
	//

	LoadTranslations("vip_modules_description.phrases");
	HookEvent("bomb_exploded", Event_bomb_exploded, EventHookMode_PostNoCopy);

	for (new client = 0; ++client <= MaxClients;)
	{
		OnClientDisconnect_Post(client);
		
		if (IsClientInGame(client)) {
			OnClientPutInServer(client);
		}
	}
}

public Event_bomb_exploded(Handle:event, const String:name[], bool:silent)
{
	for (new client = 0; ++client <= MaxClients;) {
		if (g_bAccess[client][dmg_world] && IsClientInGame(client) && (GetClientTeam(client) == 3) && IsPlayerAlive(client)) {
			
			SetEntProp(client, Prop_Send, "m_bIsDefusing", false);
			SetEntProp(client, Prop_Send, "m_iProgressBarDuration", 0);
		}
	}
}

///////////////////////////////////////////////////////////////////////////////////

public bool:VIP_CurrentItemValue(client, ItemID, String:ItemValue[], ItemValueSize)
{
	new VipModule:module = ItemIDToModule(ItemID);
	if (g_bAccess[client][module])
	{
		IntToString(g_bAccess[client][module], ItemValue, ItemValueSize);
		return true;
	}
	ItemValue[0] = 0;
	return false;
}

public bool:VIP_Description(client, ItemID, String:ItemValue[ITEM_INFO_LENGTH], String:description[], description_size)
{
	new VipModule:module = ItemIDToModule(ItemID);
	if (module == dmg_attack || module == dmg_protect)
	{
		FormatEx(description, description_size, "%T", g_sItemName[module], client, StringToInt(ItemValue));
		return true;
	}
	return false;
}

public VipGiveAction:VIP_GiveAccess(client, ItemID, String:ItemValue[ITEM_INFO_LENGTH], bool:bCallAfterAdminAction, CURRENT_ACCESS, Handle:kv)
{
	new VipModule:module = ItemIDToModule(ItemID);
	
	//  
	if (module == dmg_attack || module == dmg_protect) {
		return UpdateAccess(client, ItemValue, module);
	}
	
	// 
	if (g_bAccess[client][module]) return VGA_AlreadyEnabled;
	g_bAccess[client][module] = 1;
	VIP_NotifyItemStatusChanged(client, g_sItemName[module], true);
	
	return VGA_Continue;
}

public VIP_TakeAccess(client, ItemID, bool:bToggledByClient)
{
	new VipModule:module = ItemIDToModule(ItemID);
	g_bAccess[client][module] = 0;
	VIP_NotifyItemStatusChanged(client, g_sItemName[module], false);
}

VipGiveAction:UpdateAccess(client, String:ItemValue[ITEM_INFO_LENGTH], VipModule:module)
{
	if (!ItemValue[0]) {
		return VGA_ValueNotSpecified;
	}
	
	new value = StringToInt(ItemValue);
	if (value < 1) {
		return VGA_BadValue;
	}
	
	if (module == dmg_attack && value > 10000) {
		value = 10000;
	}
	else if (module == dmg_protect && value > 100) {
		value = 100;
	}
	
	if (value == g_bAccess[client][module]) {
		return VGA_SameValue;
	}
	
	g_bAccess[client][module] = value;
	
	IntToString(value, ItemValue, ITEM_INFO_LENGTH);
	VIP_NotifyItemStatusChanged(client, g_sItemName[module], true, ItemValue, 16);

	return VGA_Continue;
}

///////////////////////////////////////////////////////////////////////////////////

public Action:OnTakeDamage(victim, &attacker, &inflictor, &Float:damage, &damagetype)
{
	if (	((damagetype & DMG_FALL) && g_bAccess[victim][dmg_fall])
		|| 	((victim == attacker) && g_bAccess[victim][dmg_self])		)
	{
		return Plugin_Handled;
	}

	static Float:fDmg;
	fDmg = damage;

	if (attacker < 1 || attacker > MaxClients)
	{
		if (g_bAccess[victim][dmg_world])
			return Plugin_Handled;
	}
	else
	{
		if (	g_bAccess[victim][dmg_team]
			&& 	IsClientInGame(attacker)
			&& 	(GetClientTeam(attacker) == GetClientTeam(victim))	)
		{
			return Plugin_Handled;
		}

		if (g_bAccess[attacker][dmg_attack]) {
			fDmg += (fDmg / 100.0 * float(g_bAccess[attacker][dmg_attack]));
		}
	}

	if (g_bAccess[victim][dmg_protect]) {
		fDmg -= (fDmg / 100.0 * float(g_bAccess[victim][dmg_protect]));
	}

	if (fDmg != damage)
	{
		damage = fDmg;
		return Plugin_Changed;
	}

	return Plugin_Continue;
}

VipModule:ItemIDToModule(ItemID)
{
	for (new module = -1; ++module < VipModule_COUNT;) {
		if (g_ItemID[module] == ItemID) {
			return VipModule:module;
		}
	}
	return VipModule:0;
}

public OnClientPutInServer(client)
{
	SDKHook(client, SDKHook_OnTakeDamage, OnTakeDamage);
}

public OnClientDisconnect_Post(client)
{
	for (new module = -1; ++module < VipModule_COUNT;) {
		g_bAccess[client][module] = 0;
	}
}